//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * ベクトル定数補間ヘッダ
 * @author Junpee
 */

#ifndef VECTOR_CONSTANT_INTERPOLATOR_H_
#define VECTOR_CONSTANT_INTERPOLATOR_H_

#include <Animation/VectorInterpolator/VectorInterpolator.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * ベクトル定数補間
 */
class VectorConstantInterpolator : public VectorInterpolator{
public:
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 */
	VectorConstantInterpolator() : value_(Vector3::zero), length_(0.f){}

	/**
	 * デストラクタ
	 */
	virtual ~VectorConstantInterpolator(){}

	/**
	 * コピーコンストラクタ
	 * @param copy コピー元
	 */
	explicit VectorConstantInterpolator(
		const VectorConstantInterpolator& copy){
		value_ = copy.value_;
		length_ = copy.length_;
	}

	/**
	 * 代入演算子
	 * @param copy 代入元
	 */
	virtual VectorConstantInterpolator& operator =(
		const VectorConstantInterpolator& copy){
		// 自分自身ならリターン
		if(this == &copy){ return *this; }
		value_ = copy.value_;
		length_ = copy.length_;
		return *this;
	}

	//--------------------------------------------------------------------------
	/**
	 * 複製
	 * @return 複製されたベクトル補間。呼び出し元でdeleteする必要がある
	 */
	virtual VectorInterpolator* duplicate() const{
		VectorInterpolator* result = new VectorConstantInterpolator(*this);
		return result;
	}

	//--------------------------------------------------------------------------
	/**
	 * 同じ値かどうか
	 * @param target 比較対象
	 * @return 同じ値ならtrueをかえす
	 */
	virtual bool equals(const VectorInterpolator& target) const{
		VectorConstantInterpolator* interpolator =
			target.castVectorConstantInterpolator();
		if(interpolator == NULL){ return false; }
		return ((value_ == interpolator->value_) &&
			(length_ == interpolator->length_));
	}

	//--------------------------------------------------------------------------
	// 長さ
	//--------------------------------------------------------------------------
	/**
	 * 長さの取得
	 * @return 長さ
	 */
	virtual float getLength() const{ return length_; }

	/**
	 * 長さの設定
	 * @param length 長さ
	 */
	virtual void setLength(float length){
		Assert(length >= 0.f);
		length_ = length;
	}

	//--------------------------------------------------------------------------
	// バウンディング
	//--------------------------------------------------------------------------
	/**
	 * バウンディングボックスの取得
	 * @return バウンディングボックス
	 */
	virtual AxisAlignedBox getBoundingBox() const{
		AxisAlignedBox result;
		result.set(value_, value_);
		return result;
	}

	//--------------------------------------------------------------------------
	// 補間
	//--------------------------------------------------------------------------
	/**
	 * 補間
	 * @param time 時間
	 * @return 補間されたベクトル
	 */
	virtual Vector3 interpolate(float time){ return value_; }

	//--------------------------------------------------------------------------
	// 値
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param value 設定する値
	 */
	virtual void setValue(const Vector3& value){ value_ = value; }

	/**
	 * 値の取得
	 * @return 取得した値
	 */
	virtual Vector3 getValue() const{ return value_; }

	//--------------------------------------------------------------------------
	/**
	 * ベクトル定数補間かどうか
	 * @return ベクトル定数補間ならtrue
	 */
	virtual bool isVectorConstantInterpolator() const{ return true; }

	//--------------------------------------------------------------------------
private:
	// 定数
	Vector3 value_;
	// 長さ
	float length_;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of VECTOR_CONSTANT_INTERPOLATOR_H_
//------------------------------------------------------------------------------

